<?php



namespace App\Http\Controllers\api;



use App\Http\Controllers\Controller;

use Illuminate\Http\Request;

use App\Models\User;
use App\Models\Category;
use App\Models\Level;

use Illuminate\Support\Facades\Validator;

use Illuminate\Support\Facades\Hash;

use Illuminate\Support\Facades\DB;

use Illuminate\Support\Facades\Auth;


use Mail;



class UserController extends Controller

{

    public function getLevels(Request $request)
{
    $levels = Level::where('isActive', 1)
        ->select('id', 'puzzle_category', 'level_name', 'grid', 'words')
        ->get()
        ->map(function($level) {
            return [
                'id' => $level->id,
                'puzzle_category' => $level->puzzle_category,
                'level_name' => $level->level_name,
                'grid' => json_decode($level->grid),
                'words' => json_decode($level->words)
            ];
        });

    if ($levels->count() > 0) {
        return response()->json([
            'status' => 1,
            'message' => 'Levels retrieved successfully',
            'result' => $levels
        ], 200);
    }

    return response()->json([
        'status' => 0,
        'message' => 'No levels found'
    ], 404);
}


    public function Register(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'userName' => 'required',
            'email' => 'required|email',
            'password' => 'required|min:6'
        ]);
    
        if ($validator->fails()) {
            return response()->json(['error' => $validator->errors()], 401);
        }
    
        // Generate a unique token
        $userToken = md5(uniqid(rand(), true));
    
        // Create user with required fields
        $user = User::create([
            'email' => $request->email,
            'userName' => $request->userName,
            'password' => bcrypt($request->password),
            'userToken' => $userToken,
            'isActive' => 1
        ]);
    
        if ($user) {
            $response = [
                'userName' => $user->userName,
                'email' => $user->email,
                'userToken' => $user->userToken
            ];
    
            return response()->json([
                'status' => 1,
                'message' => 'User Registered Successfully',
                'result' => $response
            ], 200);
        }
    
        return response()->json([
            'status' => 0,
            'message' => 'Registration failed'
        ], 400);
    }    

    /*login*/

    public function login(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'userName' => 'required',
            'password' => 'required'
        ]);

        if ($validator->fails()) {
            return response()->json(['error' => $validator->errors()], 401);
        }

        $user = User::where('userName', $request->userName)
            ->where('isActive', 1)
            ->first();

        if ($user && Hash::check($request->password, $user->password)) {
            // Generate new token
            $userToken = md5(uniqid(rand(), true));
            $user->update(['userToken' => $userToken]);

            $response = [
                'userName' => $user->userName,
                'email' => $user->email,
                'userToken' => $userToken
            ];

            return response()->json([
                'status' => 1,
                'message' => 'Login successful',
                'result' => $response
            ], 200);
        }

        return response()->json([
            'status' => 0,
            'message' => 'Invalid credentials'
        ], 401);
    }    
    
    public function addCategory(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'categoryName' => 'required|string|max:255|unique:categories'
        ]);
    
        if ($validator->fails()) {
            return response()->json(['error' => $validator->errors()], 401);
        }
    
        $category = Category::create([
            'categoryName' => $request->categoryName,
            'categoryImage' => null // Optional: Handle image upload if needed
        ]);
    
        if ($category) {
            return response()->json([
                'status' => 1,
                'message' => 'Category added successfully'
            ], 200);
        }
    
        return response()->json([
            'status' => 0,
            'message' => 'Failed to add category'
        ], 400);
    }
    
    



    /* forgotpassword api */

    public function forgotPassword(Request $request)

    {

        $validator = Validator::make($request->all(), [

            'email' => 'required | email',

        ]);

        if ($validator->fails()) {

            return response()->json(['error' => $validator->errors()], 401);
        }

        //-------------------Message------------------------------

        $msg1 = "Your new password sent in your email";

        $msg2 = "User Not Registered with this email";

        //-------------------Message------------------------------



        $checkEmail = User::query()

            ->where('email', $request['email'])

            ->get();



        if (count($checkEmail) > 0) {

            $num = rand(11111111, 99999999);

            $update['password'] = bcrypt($num);



            $user = User::where('id', $checkEmail[0]->id)->update($update);



            if ($user) {

                $to = $request->email;

                $from = "keshavtest8@gmail.com";

                $line1 = "You forgot your password ?";

                $line2 = "No problem, use this temporary Password : " . $num;

                $subject = "Forgot password";



                $headers = "From: $from";

                $headers = "From: " . $from . "\r\n";

                $headers .= "Reply-To: " . $from . "\r\n";

                $headers .= "MIME-Version: 1.0\r\n";

                $headers .= "Content-Type: text/html; charset=UTF-8\r\n";



                $body = "<!DOCTYPE html><html lang='en'><head><meta charset='UTF-8'><title>Express Mail</title></head><body>";

                $body .= "<table style='width: 100%;'>";

                $body .= "<thead style='text-align: center;'><tr><td style='border:none;' colspan='2'>";

                $body .= "</td></tr></thead><tbody>";

                $body .= "<tr><td style='border:none;' align = 'center'>{$line1}</td></tr>";

                $body .= "<tr><td></td></tr>";

                $body .= "<tr><td style='border:none;'> </td></tr>";

                $body .= "<tr><td></td></tr>";

                $body .= "<tr><td colspan='2' style='border:none;' align = 'center'>{$line2}</td></tr>";

                $body .= "</tbody></table>";

                $body .= "</body></html>";



                mail($to, $subject, $body, $headers);





                return response()->json(['status' => 1, 'message' => $msg1]);
            }
        } else {

            return response()->json(['status' => 1, 'message' => $msg2]);
        }
    }



    /* change password api */

    public function changepassword(Request $request)
{
    $validator = Validator::make($request->all(), [
        'userId' => 'required',
        'userToken' => 'required',
        'oldPassword' => 'required',
        'newPassword' => 'required|min:6',
        'confirmPassword' => 'required|same:newPassword'
    ]);

    if ($validator->fails()) {
        return response()->json(['error' => $validator->errors()], 401);
    }

    $user = User::find($request->userId);
    
    if (!$user || $user->userToken !== $request->userToken) {
        return response()->json([
            'status' => 0,
            'message' => 'Authentication Failed'
        ]);
    }

    if (!Hash::check($request->oldPassword, $user->password)) {
        return response()->json([
            'status' => 0,
            'message' => 'Current password is incorrect'
        ]);
    }

    $user->password = Hash::make($request->newPassword);
    $user->save();

    return response()->json([
        'status' => 1,
        'message' => 'Password Updated Successfully'
    ]);
}




    public function getUserProfile(Request $request)

    {

        $validator = Validator::make($request->all(), [

            'userId' => 'required',

            'userToken' => 'required',

        ]);



        if ($validator->fails()) {

            return response()->json(['error' => $validator->errors()], 401);
        }



        $input = $request->all();

        $userId = $input['userId'];



        //-------------------Message------------------------------

        $msg1 = "User Found";

        $msg3 = "User Not Found";

        $msg2 = "You are not authorized please login again";

        //-------------------Message------------------------------  



        $verifiedUserId = Controller::checkToken($input['userId'], $input['userToken']);

        if ($verifiedUserId == $input['userId']) {

            $user = User::query()->where('id', '=', $input['userId'])->where('userType', '=', 'Customer')->where('status', '=', 1)->get();



            if (count($user) > 0) {

                $userImg = "";

                $picture_value = $user[0]->profileImage;

                if ($picture_value != "" && $picture_value != null) {

                    if (file_exists(public_path() . "/uploads/customerImage/" . $picture_value)) {

                        $userImg = asset('uploads/customerImage/' . $picture_value);
                    }
                }





                $userData['userId'] = $user[0]->id;

                $userData['userToken'] = $user[0]->userToken;

                $userData['fullName'] = $user[0]->fullName;

                $userData['email'] = $user[0]->email;

                $userData['phoneNumber'] = $user[0]->phoneNumber;

                $userData['userImage'] = $userImg;

                $userData['location'] = $user[0]->location;

                $userData['latitude'] = $user[0]->latitude;

                $userData['longitude'] = $user[0]->longitude;

                $userData['country'] = $user[0]->country;

                $userData['state'] = $user[0]->state;

                $userData['city'] = $user[0]->city;

                $userData['postal_code'] = $user[0]->postal_code;

                $userData['userType'] = $user[0]->userType;



                return response()->json(['status' => 1, 'message' => $msg1, 'result' => $userData]);
            } else {

                return response()->json(['status' => 1, 'message' => $msg2]);
            }
        } else {

            return response()->json(['status' => 0, 'message' => $msg3]);
        }
    }
}
