<?php

namespace App\Http\Controllers\adminpanel;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Category;
use App\Models\Category_level;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\File;

class CategoryController extends Controller
{
    private $pagination = 20;
    public function manage()
    {
        $data = Category_level::query()->paginate($this->pagination);
        return view("adminpanel.managecategory", compact("data"));
    }

    public function search(Request $request)
    {
        $input = $request->all();
        $qry = Category_level::query();

        if(trim($input["search"]) != ""){
            $search = $input['search'];
            $qry->where([
                ["categoryName", "like", "%{$search}%"],
            ]);
        }

        $data = $qry->paginate($this->pagination);
        return view("adminpanel.managecategory", compact("data"));
    }

    public function add()
    {
        $data = array('type' => 'add');
        return view("adminpanel.addcategory", compact("data"));
    }

    public function save(Request $request)
    {
        $input = $request->all();


        if(isset($input['categoryImage'])){
            $upload_dir_path = public_path()."/uploads/categoryImage";
            $imagePath = $input['categoryImage'];
            $extension = pathinfo($imagePath->getClientOriginalName(), PATHINFO_EXTENSION);
            $filename = time()."categoryImage.". $extension;
            $imagePath->move($upload_dir_path, $filename);
            $input['categoryImage'] = $filename;
        }
        $input['categoryName'] = $input['categoryName'];
        $input['display_name'] = $input['categoryName'];
        $input['save_id'] = rand(0000,9999).''.time();
        $input['lock_type'] = $input['lock_type'];
        $input['unlock_amount'] = $input['unlock_amount'];
        $input['category_color'] = $input['category_color'];

        $category = Category_level::create($input);
        if($category->id>0){
            return redirect()->route('adminpanel.category.manage')->with('success','Category Created Successfully');
        }
        else{
            return redirect()->route('adminpanel.category.add')->withErrors(["Error creating record. Please Try Again"]);
        }
    }

    public function edit($id)
    {
        $input = Category_level::where('id','=',$id)->get();
        $data = array("type"=>'edit',"input"=>$input);

        return view("adminpanel.addcategory", compact("data"));
    }

    public function update(Request $request)
    {
        $input = $request->all();
        $id = $input['id'];

        $update = array();

        if (!empty($input['categoryName'])) {
            $update['categoryName'] = $input['categoryName'];
        }
        $update['display_name'] = $input['categoryName'];
        $update['lock_type'] = $input['lock_type'];
        $update['unlock_amount'] = $input['unlock_amount'];
        $update['category_color'] = $input['category_color'];

        if(isset($input['categoryImage'])){
            $upload_dir_path = public_path()."/uploads/categoryImage";
            $imagePath = $input['categoryImage'];
            $extension = pathinfo($imagePath->getClientOriginalName(), PATHINFO_EXTENSION);
            $filename = time()."categoryImage.". $extension;
            $removeImage = $this->removeImage($upload_dir_path, $id);
            $imagePath->move($upload_dir_path, $filename);
            $update['categoryImage'] = $filename;
        }
        $user = Category_level::where('id','=',$id)->update($update);
        return redirect()->route('adminpanel.category.manage')->with('success', 'Category Updated Successfully');
    }

    private function removeImage($upload_dir_path, $id)
    {
        $image = Category_level::where('id','=',$id)->get();
        // dd($image[0]->categoryImage);
        if($image[0]->categoryImage!=null && $image[0]->categoryImage!="")
        {
            if(File::exists($upload_dir_path.'/', $image[0]->categoryImage)){
                unlink($upload_dir_path.'/'. $image[0]->categoryImage);
            }
        }
        return true;
    }

    public function delete($id)
    {
        Category_level::where('id','=',$id)->delete();
        return redirect()->route('adminpanel.category.manage')->with('success','Category Deleted Succesfully');
    }

    public function active($id)
    {
        $update['isActive'] = 1;
        Category_level::where('id','=',$id)->update($update);
        return redirect()->route('adminpanel.category.manage')->with('success','Active Successfully');
    }

    public function deactive($id)
    {
        $update['isActive'] = 0;
        Category_level::where('id','=',$id)->update($update);
        return redirect()->route('adminpanel.category.manage')->with('success','Deactive Successfully');
    }

    private function getRules($type, $input)
    {
        $return = array();
        $return['categoryName'] = "required";
        return $return;
    }

    private function messages()
    {
        return [
            "categoryName.required" => $this->getRequiredMessage("Category Name"),
        ];
    }

    private function getRequiredMessage($string)
    {
        return "The ". $string ." field is required";
    }
}
