<?php

namespace App\Http\Controllers\adminpanel;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Category;
use App\Models\Category_level;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\File;

class CategoryLevelController extends Controller
{
    private $pagination = 20;
    public function manage()
    {
        $data = Category_level::query()->paginate($this->pagination);
        return view("adminpanel.managecategorylevel", compact("data"));
    }

    public function search(Request $request)
    {
        $input = $request->all();
        $qry = Category_level::query();

        if(trim($input["search"]) != ""){
            $search = $input['search'];
            $qry->where([
                ["display_name", "like", "%{$search}%"],
            ]);
        }

        $data = $qry->paginate($this->pagination);
        return view("adminpanel.managecategorylevel", compact("data"));
    }

    public function add()
    {
        $data = array('type' => 'add');
        $category = Category::get()->pluck('categoryName', 'id')->toArray();

        return view("adminpanel.addcategorylevel", compact("data", 'category'));
    }

    public function save(Request $request)
    {
        $input = $request->all();

        $input['category_id'] = $input['category_id'];
        $input['display_name'] = $input['display_name'];
        $input['save_id'] = rand(0000,9999);
        $input['lock_type'] = $input['lock_type'];
        $input['unlock_amount'] = $input['unlock_amount'];
        $input['category_color'] = $input['category_color'];

        $category = Category_level::create($input);
        if($category->id>0){
            return redirect()->route('adminpanel.categorylevel.manage')->with('success','Category Created Successfully');
        }
        else{
            return redirect()->route('adminpanel.categorylevel.add')->withErrors(["Error creating record. Please Try Again"]);
        }
    }

    public function edit($id)
    {
        $input = Category_level::where('id','=',$id)->get();
        $data = array("type"=>'edit',"input"=>$input);
        $category = Category::get()->pluck('categoryName', 'id')->toArray();

        return view("adminpanel.addcategorylevel", compact("data", 'category'));
    }

    public function update(Request $request)
    {
        $input = $request->all();
        $id = $input['id'];

        $category = Category::get()->pluck('categoryName', 'id')->toArray();

        $validator = Validator::make( $input, $this->getRules('Edit', $input), $this->messages());

        if ($validator->fails()) {
            $data = array('type'=>'Edit', 'input'=>$input, 'error'=>$validator->messages());
            return view('adminpanel.addcategorylevel', compact('data', 'category'));
            exit();
        }

        $update = array();

        $update['category_id'] = $input['category_id'];
        $update['display_name'] = $input['display_name'];
        $update['lock_type'] = $input['lock_type'];
        $update['unlock_amount'] = $input['unlock_amount'];
        $update['category_color'] = $input['category_color'];

        $user = Category_level::where('id','=',$id)->update($update);
        return redirect()->route('adminpanel.categorylevel.manage')->with('success', 'Category Updated Successfully');
    }

    public function delete($id)
    {
        Category_level::where('id','=',$id)->delete();
        return redirect()->route('adminpanel.categorylevel.manage')->with('success','Category Deleted Succesfully');
    }

    public function active($id)
    {
        $update['isActive'] = 1;
        Category_level::where('id','=',$id)->update($update);
        return redirect()->route('adminpanel.categorylevel.manage')->with('success','Active Successfully');
    }

    public function deactive($id)
    {
        $update['isActive'] = 0;
        Category_level::where('id','=',$id)->update($update);
        return redirect()->route('adminpanel.categorylevel.manage')->with('success','Deactive Successfully');
    }

    private function getRules($type, $input)
    {
        $return = array();
        $return['display_name'] = "required";
        return $return;
    }

    private function messages()
    {
        return [
            "display_name.required" => $this->getRequiredMessage("Category Name"),
        ];
    }

    private function getRequiredMessage($string)
    {
        return "The ". $string ." field is required";
    }
}
