<?php

namespace App\Http\Controllers\adminpanel;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Category;
use App\Models\Category_level;
use App\Models\Level;
use App\Models\Levelword;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\File;
use Illuminate\Http\JsonResponse;

class LevelController extends Controller
{
    private $pagination = 20;
    public function manage()
    {
        $data = Level::query()->paginate($this->pagination);
        return view("adminpanel.managelevel", compact("data"));
    }

    public function search(Request $request)
    {
        $input = $request->all();
        $qry = Level::query();

        if(trim($input["search"]) != ""){
            $search = $input['search'];
            $qry->where([
                ["display_name", "like", "%{$search}%"],
            ]);
        }

        $data = $qry->paginate($this->pagination);
        return view("adminpanel.managelevel", compact("data"));
    }

    public function add()
    {
        $data = array('type' => 'add');
        $objectionableWords = Levelword::first()->levelWord ?? '';
        $category = Category_level::get()->pluck('categoryName', 'id')->toArray();

        return view("adminpanel.addlevel", compact("data", 'category','objectionableWords'));
    }

    public function save(Request $request)
    {
        $input = $request->all();
    
        // Parse the JSON data
        $jsonData = json_decode($input['data'], true);
    
        // Create level data array
        $levelData = [
            'category_id' => $input['category_id'],
            'puzzle_category' => $input['category_id'],
            'level_name' => $input['display_name'],
            'display_name' => $input['display_name'],
            // 'unlock_amount' => $input['unlock_amount'],
            // 'lock_type' => $input['lock_type'],
            // 'category_color' => $input['category_color'],
            'grid' => json_encode($jsonData['board']), // Store board as JSON
            'words' => json_encode($jsonData['words']), // Store words as JSON
            'clues' => json_encode($jsonData['clues']), 
            'isActive' => 1,
            'data' => $input['data'] // Store complete JSON data
        ];
    
        // Save to database
        $level = Level::create($levelData);
    
        if ($level->id > 0) {
            return redirect()->route('adminpanel.level.manage')
                            ->with('success', 'Level Created Successfully');
        }
    
        return redirect()->route('adminpanel.level.add')
                        ->withErrors(['Error creating record']);
    }
    
    

    public function edit($id) 
    {
        $level = Level::findOrFail($id);
        
        $data = [
            "type" => 'edit',
            "input" => $level
        ];
        
        $category = Category_level::pluck('categoryName', 'id')->toArray(); 
        
        // Decode JSON data
        $display_name = $level->level_name;
        
        // Fetch category name safely
        $puzzle_category = $level->category ? $level->category->categoryName : 'Unknown Category';
        
        $gridData = json_decode($level->data, true);
        $gridArray = json_decode($level->grid, true);
        $wordList = implode(',', json_decode($level->words, true));
    
        // print_r($puzzle_category);
        // exit();
    
        return view("adminpanel.addlevel", compact(
            'data',
            'category',
            'level',
            "display_name",
            'puzzle_category',
            'gridData',
            'gridArray',
            'wordList'
        ));
    }
    

    
    
    public function update(Request $request)
    {
        $input = $request->all();
        $id = $input['id'];
    
        // Parse the JSON data
        $jsonData = json_decode($input['data'], true);
        
        // Prepare update data
        $updateData = [
            'puzzle_category' => $input['category_id'],
            'level_name' => $input['display_name'],
            'grid' => json_encode($jsonData['board']),
            'words' => json_encode($jsonData['words']),
            'clues' => json_encode($jsonData['clues']), 
            'data' => $input['data']
        ];


        // print_r($id);   
        // exit();
    
        Level::where('id', $id)->update($updateData);
        return redirect()->route('adminpanel.level.manage')->with('success', 'Level Updated Successfully');
    }
    
    

    public function delete($id)
    {
        Level::where('id','=',$id)->delete();
        return redirect()->route('adminpanel.level.manage')->with('success','Category Deleted Succesfully');
    }

    public function active($id)
    {
        $update['isActive'] = 1;
        Level::where('id','=',$id)->update($update);
        return redirect()->route('adminpanel.level.manage')->with('success','Active Successfully');
    }

    public function deactive($id)
    {
        $update['isActive'] = 0;
        Level::where('id','=',$id)->update($update);
        return redirect()->route('adminpanel.level.manage')->with('success','Deactive Successfully');
    }

    public function getGlobalForbiddenWords(): JsonResponse
    {
        $globalForbiddenWords = Levelword::first();

        if ($globalForbiddenWords && $globalForbiddenWords->levelWord) {
            return response()->json($globalForbiddenWords->levelWord);
        }

        return response()->json('');
    }

    private function getRules($type, $input)
    {
        return [
            'level_name' => 'required',  // Changed from display_name to level_name
        ];
    }

    private function messages()
    {
        return [
            'level_name.required' => $this->getRequiredMessage('Level Name'),
            'puzzle_category.required' => $this->getRequiredMessage('Category')
        ];
    }

    private function getRequiredMessage($string)
    {
        return "The ". $string ." field is required";
    }

    public function ajaxgetLevelCategory($id)
    {
        print_r($id);
        exit();
        $arr['data'] = Category_level::Where('category_id','=',$id)->get(); 
        echo json_encode($arr);
    }
}
