<?php



namespace App\Http\Controllers\adminpanel;



use App\Http\Controllers\Controller;

use Illuminate\Http\Request;

use App\Models\Subcategory;

use App\Models\Category;

use Validator;

use Illuminate\Support\Str;



use Illuminate\Pagination\Paginator;



class SubCategoryController extends Controller

{

    private $pagination = 20;


    public function manage()
    {
        $data = Subcategory::query()->paginate($this->pagination);
        $cat = Category::where('isActive', '=', 1)->get()->pluck('categoryName', 'id')->toArray();

        return view('adminpanel.managesubcategory', compact('data', 'cat'));
    }




    public function add()

    {

        $cat = Category::where('isActive', '=', 1)->get()->pluck('categoryName', 'id')->toArray();

        $data = array('type' => 'add');

        return view('adminpanel.addsubcategory', compact('data', 'cat'));
    }



    public function save(Request $request)
    {
        $input = $request->all();
        $validator = Validator::make($input, $this->getRules('Add', $input), $this->messages());

        if ($validator->fails()) {
            $cat = Category::where('isActive', '=', 1)->get()->pluck('categoryName', 'id')->toArray();
            $data = array('type' => 'add', 'input' => $input, 'error' => $validator->messages());
            return view('adminpanel.addsubcategory', compact('data', 'cat'));
        }

        $input['categoryId'] = $input['categoryId'];
        $input['subcategoryName'] = $input['subcategoryName'];

        $category = Subcategory::create($input);

        if ($category->id > 0) {
            return redirect()->route('adminpanel.subcategory.manage')->with('success', 'Created successfully.');
        }

        return redirect()->route('adminpanel.subcategory.add')->withErrors(['Error creating record. Please try again.']);
    }



    public function edit($id)

    {

        $cat = Category::where('isActive', '=', 1)->get()->pluck('categoryName', 'id')->toArray();


        $input = Subcategory::where('id', '=', $id)->get();

        $data = array('type' => 'Edit', 'input' => $input);

        return view('adminpanel.addsubcategory', compact('data', 'cat'));
    }



    public function update(Request $request)

    {

        //dd($request->all());



        $input = $request->all();

        $id = $input['id'];

        $validator = Validator::make($input, $this->getRules('Edit', $input), $this->messages());



        if ($validator->fails()) {

            $data = array('type' => 'Edit', 'input' => $input, 'error' => $validator->messages());

            return view('adminpanel.addsubcategory', compact('data'));

            exit();
        }

        $update = array();



        /*if(isset($input["categoryImage"])) 

        {

            $upload_dir_path = public_path()."/uploads/Category_images";

            $this->removeimage($upload_dir_path, $id);

            

            $categoryImage = $input["categoryImage"];

            $filename = rand(0000,9999).$categoryImage->getClientOriginalName();

            

            $categoryImage->move($upload_dir_path, $filename );

            $update['categoryImage'] = $filename;

        }*/



        $update['categoryId'] = $input['categoryId'];

        $update["subcategoryName"] = $input['subcategoryName'];




        $category = Subcategory::where('id', '=', $id)->update($update);

        return redirect()->route('adminpanel.subcategory.manage')->with('success', 'Updated successfully.');
    }



    public function delete($id)

    {

        // $upload_dir_path = public_path()."/uploads/Category_images";

        // $this->removeimage($upload_dir_path, $id);



        Subcategory::where('id', '=', $id)->delete();

        return redirect()->route('adminpanel.subcategory.manage')->with('success', 'Deleted successfully.');
    }



    /*private function removeimage($imagepath, $id)

    {

        $user = Category::where('id', '=', $id)->get();

        if($user[0]->categoryImage!=null && $user[0]->categoryImage!="")

        {       

            if(file_exists($imagepath.'/'.$user[0]->categoryImage))

            {

                unlink($imagepath.'/'.$user[0]->categoryImage);

            }

        }

        return true;

    }*/

    public function search(Request $request)
    {
        $input = $request->all();
        $qry = Subcategory::query();
        if (trim($input["search"]) != "") {
            $search = $input["search"];
            $qry->where([
                ["subcategoryName", "like", "%{$search}%"],
            ]);
        }
        $data = $qry->paginate($this->pagination);
        $data->appends($input);
        return view('adminpanel.managesubcategory', compact('data'));
    }



    private function getRules($type, $input)
    {

        $return = array();

        $return['subcategoryName'] = 'required|max:30';
        return $return;
    }



    private function messages()

    {

        return [

            'subcategoryName.required'  => $this->getRequiredMessage('subcategory name'),

            'subcategoryName.max'  => $this->getGreaterMessage('subcategory name', 30),

        ];
    }



    private function getRequiredMessage($string)
    {

        return 'The ' . $string . ' field is required.';
    }



    private function getGreaterMessage($string, $maxchar)
    {

        return 'The ' . $string . ' may not be greater than ' . $maxchar . ' characters.';
    }
}
