<?php

namespace App\Http\Controllers\adminpanel;

use App\Http\Controllers\Controller;
use App\Models\Category;
use Illuminate\Http\Request;
use Validator;
use App\Models\User;
use Dotenv\Validator as DotenvValidator;
use Illuminate\Pagination\Paginator;
use Mail;

class UserController extends Controller
{
    private $pagination = 20;
    public function manage()
    {
        $data = User::query()
            // ->where('userType',1)

            ->paginate($this->pagination);

        return view('adminpanel.manageuser', compact('data'));
    }
    public function add()
    {
        $data = array('type' => 'add');
        return view('adminpanel.adduser', compact('data'));
    }

    public function save(Request $request)
    {
        $this->validate($request,[
            'userName' => 'required',
            'email' => 'required|email|unique:users',
            'password' => 'required',
        ]);
    
        $input = $request->only(['userName', 'email']);
        $input['password'] = bcrypt($request->password);
        
        // Generate unique token
        $input['userToken'] = md5(uniqid(rand(), true));
        
        $user = User::create($input);
        
        if($user->id > 0) {
            return redirect()->route('adminpanel.user.manage')->with('success', 'User Created successfully.');
        }
        return redirect()->route('adminpanel.user.add')->withErrors(['Error creating User. Please try again.']);
    }
    

    public function edit($id)
    {
        $input = User::where('id', '=', $id)->get();
        $data = array('type' => 'Edit', "input" => $input,);
        return view('adminpanel.adduser', compact('data'));
    }
    public function update(Request $request)
    {
        $this->validate($request,[
            'userName' => 'required',
            'email' => 'required|email|unique:users,email,'.$request->id,
        ]);
    
        $input = $request->only(['userName', 'email']);
        
        if($request->filled('password')) {
            $input['password'] = bcrypt($request->password);
        }
        
        $user = User::where('id', '=', $request->id)->update($input);
        
        return redirect()->route('adminpanel.user.manage')
                ->with('success', 'User Updated successfully.');
    }
    public function delete($id)
    {
        User::where('id', '=', $id)->delete();
        return redirect()->route('adminpanel.user.manage')->with('success', 'User Deleted successfully.');
    }
    public function active($id)
    {
        $update['isActive'] = 1;
        User::where('id', '=', $id)->update($update);
        return redirect()->route('adminpanel.user.manage')->with('success', 'Active successfully.');
    }
    public function deactive($id)
    {
        $update['isActive'] = 0;
        User::where('id', '=', $id)->update($update);
        return redirect()->route('adminpanel.user.manage')->with('success', 'Deactive successfully.');
    }
    public function search(Request $r)
    {
        $input = $r->all();
        $qry = User::query(); // Initialize the query builder
        
        if(trim(isset($input["search"])) && $input["search"] != "") {
            $search = $input["search"];
            
            $qry->where(function($query) use ($search) {
                $query->where('email', 'like', "%{$search}%")
                      ->orWhere('userName', 'like', "%{$search}%");
            });
        }
        
        $data = $qry->paginate($this->pagination);
        $data->appends($input);
        
        return view('adminpanel.manageuser', compact('data'));
    }
    
    /* ================================================================================== */
    private function removeimage($imagepath, $id)
    {
        $user = User::where('id', '=', $id)->get();
        if ($user[0]->profileImage != null && $user[0]->profileImage != "") {
            if (file_exists($imagepath . '/' . $user[0]->profileImage)) {
                unlink($imagepath . '/' . $user[0]->profileImage);
            }
        }
        return true;
    }
    private function removefimage($imagepath, $id)
    {
        $user = User::where('id', '=', $id)->get();
        if ($user[0]->fprofileImage != null && $user[0]->fprofileImage != "") {
            if (file_exists($imagepath . '/' . $user[0]->fprofileImage)) {
                unlink($imagepath . '/' . $user[0]->fprofileImage);
            }
        }
        return true;
    }
    private function getRules($type, $input, $userId = '0')
    {
        $return = array();
        $return['userName'] = 'required';
        $return['phone']    = 'required|regex:/^([0-9\s\-\+\(\)]*)$/|min:10|max:13';
        if ($type == "Edit") {
            //$return['email'] = 'required|email|unique:users,email|max:100';
            $return['email'] = 'required|email|unique:users,email,' . $userId . '|max:100';
        } else {
            $return['email']        = 'required|email|unique:users,email|max:100';
            // $return['password']     = 'required|min:6|max:20';
            //$return['profileImage'] = 'required';
        }
        return $return;
    }
    private function messages()
    {
        return [
            'userName.required'    => $this->getRequiredMessage('User name'),
            'phone.required'        => $this->getRequiredMessage('phone no.'),
            'phone.min'             => $this->getLessMessage('phone no.', 10),
            'phone.max'             => $this->getGreaterMessage('phone no.', 13),
        ];
    }
    private function getRequiredMessage($string)
    {
        return 'The ' . $string . ' field is required.';
    }
    private function getLessMessage($string, $minchar)
    {
        return 'The ' . $string . ' may not be less than ' . $minchar . ' characters.';
    }
    private function getGreaterMessage($string, $maxchar)
    {
        return 'The ' . $string . ' may not be greater than ' . $maxchar . ' characters.';
    }
}
